// getch-plus.h

/**
 * \file getch-plus.h
 * Only two or three functions are needed to use getch(). But half of this 
 * mess is required for getch(), so why not use the whole thing.  
 * Easier to test.  -rs
 * 
 * (C) 2012, Rainbow Sally.  Released under GPL.
 */


#ifndef getch_term_h
#define getch_term_h

#include <LQ/cdecls.h>
LQ_BEGIN_CDECLS;

/// Sets one or more flags on or off in termios struct.  Used by
/// term_setEcho(on/off), etc.
void term_setLocalFlags(unsigned flags, int on_off);

/// Turns echo on or off.  Any non-zero value will be interpreted
/// as 'on'.
void term_setEcho(int on_off);

/// Turns ctrl-c break handling on or off.
void term_setCBreak(int on_off);

/// Turns echo off and sets termios to non-canonical config needed
/// by getch() and other funcs.
void term_setNonCanonical();

/// Restores original terminal configuration
void term_setCanonical();

/// returns non-zero if keyboard buffer is completely full.
int keybuf_full();

/// returns non-zero if the keyboard buffer is completely empty.
int keybuf_empty();

/// pops the next key from the keyboard buffer.  Returns 0 if none.
int pop_key();

/// extracts and returns the string at the keyboard buffer, 
/// zero terminated, possibly longer than a real ansi code.  
/// Does not remove chars from the keybuf, terminates 
/// string at next ESC char or 8 chars, whichever comes first.
/// Used internally, but may be useful externally if exact strings
/// (ansi esc sequences) need to be decoded better than the simple
/// built-in version.
const char* keybuf_toString();

/// place to plug in a background task.  This task should run quickly
/// or in parts so the keyboard doesn't hang.  The enable for this 
/// function should be checked to see if it's disabled for cases 
/// when reading keys for cursor movement, or other things that require
/// very rapid key reading.

extern void (*keybuf_bgtask)();
extern int keybuf_bgtask_enable;

/// only checks if a key is ready, has nothing to do with the keybuf
/// or term setup.
int os_keyin();

/// Used internally, returns number of bytes read.
int keybuf_update();

/// wipes out the keybuffer and also anything the OS may be waiting
/// to send.
void keybuf_clear();

/// can be set to zero if this causes conflicts with other key names.
#define USE_KEYBUF_KEYNAMES 1

#if USE_KEYBUF_KEYNAMES
enum
{
  F1_KEY = 0x100,  F2_KEY,  F3_KEY,  F4_KEY,  F5_KEY,  F6_KEY,
  F7_KEY,  F8_KEY,  F9_KEY,  F10_KEY,  F11_KEY,  F12_KEY,
  
  INS_KEY,  HOME_KEY,  PGUP_KEY,  DEL_KEY,  END_KEY,  PGDN_KEY
};

// FKEY_MASK is for function keys as well as cursor keys
#define FKEY_MASK 0x100
#define CTRL_MASK 0x200
#define ALT_MASK 0x400

#endif // USE_KEYBUF_KEYNAMES

/// updates keybuf and optionally calls user defined keybuf_keylookup()
/// or returns ascii or encoded ansi seq as a single integer value if
/// the key is a string starting with an esc char.  Funky but ok for 
/// simple jobs.  And if fact, this routine is what the entire utility
/// is about.
int keybuf_getKey();

/// Waits for a keyboard input and optionally calls a user defined 
/// background task while waiting.
void keybuf_keyWait();

/// And here it is.  The moment you've been waiting for...
int getch();

/** \file getch-term.h
  
  To use getch() use
  - term_setNonCanonical();
  to allow turning off echo and do non-blocking keyboard input.
*/


LQ_END_CDECLS;

#endif // getch_term_h

