
#include <string.h> // strlen()
#include <stdlib.h> // getenv()

#include <LQ/filename.h>
#define NO_PARSER_WARN 1
#include <LQ/Parser.h> // load last in list

#include <LQ/cdecls.h>
LQ_BEGIN_CDECLS;

// returns true if successful
bool filename_absolute (char *ret, size_t retlen, const char *filename)
{
  using namespace Parser;
  *ret = 0;
  
  // the output accumulator also used as an intermediate buffer
  int buflen;
  char *buf;
  
  // use a parser object
  PARSER_OBJ This;
  parser_init (&This, filename, strlen(filename)+1, false);
  
  // if relative to current dir.
  // prefix ../ with current dir
  if(obj_is("../"))
  {
    obj_put(getenv("PWD"));
    obj_put("/");
  }
  else
  {    
    // replace ./ with current dir
    if(obj_skip("./"))
      obj_put(getenv("PWD"));
  }
  
  obj_put(obj.ip); // don't need to clear ip because...
  
  // save result so far, including terminating null char
  // strdup is safe here because the final result will be 
  // shorter if anything, so we can use strdup to create the
  // final output buffer.
  
  buf = strdup(obj.obuf);
  buflen = strlen(obj.obuf)+1;
  parser_free(&This);
  
  // restart using previous results
  parser_init (&This, buf, buflen, false);
  
  while (obj_not_eoi ())
    // if '/..' then back up input to just above previous '/'
  {
    if(obj_skip("/.."))
    {
      while((*obj.op != '/') && (obj.op > obj.obuf))
        obj.op--;
      *obj.op = 0; // terminate for form
    }
    else
      obj_copy_nchars(1);
  }
  
  strcpy(buf, obj.obuf); // save output back to buf
  
  // free and restore from last to first like a stack
  parser_free(&This);
  if(strlen(buf) > retlen)
    return false;
  else
  {
    strcpy(ret, buf);
    return true;
  }
}

LQ_END_CDECLS;
