// tmpstr.c

#include <malloc.h> // realloc()
#include <stdio.h>  // vnsprintf()
#include <stdarg.h> // va_list()
#include <stdlib.h> // exit()
#include <LQ/tmpstr.h>

#include <LQ/cdecls.h> // LQ_BEGIN_CDECLS
LQ_BEGIN_CDECLS;

static void* tmps[TMPMEM_MAX_BUFFERS];
static int tmps_index = 0;

// get a temp block of memory of size bytes.
void* tmpmem(int size)
{
  void* buf = tmps[tmps_index];
  if(buf)
    buf = realloc(buf, size);
  else
    buf = malloc(size);
  tmps[tmps_index++] = buf;
  if(!buf) {
    fprintf(stderr, "Fatal error: Can't allocate temp buffer (tmpmem/tmpstr)\n");
    exit(1);
  }
  if(tmps_index >= TMPMEM_MAX_BUFFERS) tmps_index = 0;
  return buf;
}

  
/// Returns a tempstring from a recycling buffer up to size bytes long
char* tmpstr(int size_max, const char* fmt, ...)
{
  char* mem = (char*)tmpmem(size_max);
  if(fmt != 0)
  {
    va_list args;
    va_start(args, fmt);
    vsnprintf(mem, size_max, fmt, args);
    va_end(args);
  }
  return mem;  
}

int tmpmem_adopt(void* obj)
{
  int i; // loop index
  for(i = 0; i < TMPMEM_MAX_BUFFERS; i++) {
    if(tmps[i] == obj) {
      tmps[i] = 0;
      return 0;
    }
  } // we we get here, it's technically an error.
  return 1;
}

int tmpstr_adopt(char* obj) { return tmpmem_adopt((void*) obj);}

LQ_END_CDECLS;
