// slist.cpp -- lists of strings in the format of an args list, 0 terminated.

#include <malloc.h> // malloc and free
#include <string.h> // str*() funcs

#include "slist.h"

// returns an initialized pointer to a list base
char** slist_new()
{
  char** p = (char**)malloc(sizeof(char**));
  *p = 0;
  return p;
}


// removes item and deletes the base
void slist_delete(char** list)
{
  int i = 0;
  while(list[i])
    free(list[i++]);
  free(list);
}

// returns number of items in the list
int slist_count(char** list)
{
  int i = 0;
  while(list[i])
    i++;
  return i;
}

// removes items, does not resize or delete the base
void slist_clear(char** list)
{
  int i = 0;
  for(int i =  0; list[i] != 0; i++)
  {
    free(list[i]);
    list[i] = 0;
  }
}

// adds a new items at the end of the list
void slist_append(char*** plist, const char* s)
{
  int count;
  char** list = *plist;
  for(count = 0; list[count]; count++)
    ; // noop
  list = (char**)realloc(list, sizeof(char**) * (count + 2));
  list[count] = strdup(s);
  list[count + 1] = 0;
  *plist = list;
}

// removes (and frees) item at idx. Does not resize the list
void slist_remove(char*** plist, int idx)
{
  char** list = *plist;
  int len = slist_count(list);
  if((idx < 0) || (idx > len))
    return;
  free(list[idx]);
  for(int i = idx; i < len; i++)
    list[i] = list[i+1];
  // *plist = list; -- already set since no resizing is done
}


// adds list list2 to list1, does not delete list that's added
void slist_addList(char*** plist1, char** list2)
{
  int len = slist_count(list2);
  for(int i = 0; i < len; i++)
    slist_append(plist1, list2[i]);
}


// remove one trailing newline char from a string
static void trim_newline(char* buf)
{
  char* p = buf + strlen(buf) -1;
  if(*p == '\n')
    *p = 0;
}

// Loads list with strings from a piped command,
// returns 0 on success
int slist_readPipe(char*** plist, const char* pipecmd)
{
  size_t bufsize = 4096;
  char* buf = (char*) malloc(bufsize);
  char** list = *plist;
  FILE* fp = popen(pipecmd, "r");
  if(! fp) return 1;
  while(! feof(fp))
  {
    *buf = 0;
    getline(&buf, &bufsize, fp);
    
    // done if nothing is read
    if(! *buf)
      break;
    
    trim_newline(buf);
    slist_append(&list, buf);
  }
  fclose(fp);
  
  // save the result and return ok
  free(buf);
  *plist = list;
  return 0;
}

// writes list through pipe, returns 0 on success.
int slist_writePipe(char*** plist, const char* pipecmd)
{
  char** list = *plist;
  FILE* fp = popen(pipecmd, "w");
  if(! fp) return 1;
  for(int i = 0; i < slist_count(list); i++)
  {
    char* line = list[i]; // to see in debugger
    fprintf(fp, "%s\n", line);    
  }
  fclose(fp);
  return 0;
}


// TODO:
// slist_insert(char*** plist, s, idx)   -- insert new item at idx
// slist_moveList(plist1, idx, plist2, delete_flg)  -- overwrite and append,
//                                                 -- [empty or delete list2]
// slist_trim(plist, idx)                -- remove items from idx to end
// slist_shift(plist, n)                 -- remove first n items from list

 
