/****************************************************************** 
  This is the code for: create/qt4-files/write-files, called by 
  init.sh which just calls this app with the full path of the 
  ui file.

  In other words:

  This program does the nuts and bolts of 'mc2 -create qt4-files' 
  which will generate a simple C++ file set for a designer or a 
  qtcreator ui file which can be compiled with an mc2 makefile.  

  It will place the C++ sources in the same folder as the ui file.

  This may need to updated later, as we get into creating more 
  complex user interfaces.  QT3 allowed much greater flexibility 
  with subclassing these critters though.  Progess...  Sometimes 
  you wonder.  So it's also possible we may NOT have to update 
  this later.
  
  In any case here's an example of a program we can use as a 
  -create function from mc2.

  Note that the main function is placed in app-main.cpp so as to 
  avoid clobbering a preexisting main.cpp in case there is one.
 ******************************************************************/

#include <string.h> // strncpy()
#include <stdio.h>  // sprintf()
#include <stdlib.h> // exit()
#include <ctype.h>  // toupper(), tolower(), ...

void dbg(){}

// These dat files have '%s' inserted for each parameter
// required.  The parameters are the names of variables 
// that resemble the actual names in terms of upper/lower 
// case but are not the actual variable's values which are
// set up in main().

#include "app_main_cpp.dat"
// s1 = widgetname
// s2 = WidgetName

// These defines are just for convenience, being the parameters
// in the right order for use by the formats loaded in the dat
// files.
#define app_main_cpp_args widgetname, WidgetName

#include "widgetname_cpp.dat"
// s1 = widgetname
// s2 = widgetname
// s3 = widgetname
// s4 = WidgetName, s5 = WidgetName
// s6 = QParentClass
// s7 = WidgetName
// s8 = WidgetName, s9 = WidgetName
#define widgetname_cpp_args \
  widgetname, widgetname, widgetname, WidgetName, WidgetName, \
  QParentClass, WidgetName, WidgetName, WidgetName


#include "widgetname_h.dat"
// s1 = widgetname
// s2 = WIDGETNAME
// s3 = WIDGETNAME
// S4 = QParentClass
// s5 = WidgetName
// s6 = WidgetName S7 = QParentClass
// s8 = WidgetName
// s9 = WidgetName
// s10 = WidgetName
// s11 = WIDGETNAME

#define widgetname_h_args \
  widgetname, WIDGETNAME, WIDGETNAME, QParentClass, \
  WidgetName, WidgetName, QParentClass, \
  WidgetName, WidgetName, WidgetName, WIDGETNAME

#define error(s, what) fprintf(stderr, s, what), fprintf(stderr, "\n"), exit(1)

// These are string case converters for generating two new variables 
// from a single WidgetName variable.
static void strupper(char* dest, const char* src)
{
  while(*src)
  {
    *dest++ = toupper(*src++);
  }
  *dest = 0;
}
    
static void strlower(char* dest, const char* src)
{
  while(*src)
  {
    *dest++ = tolower(*src++);
  }
  *dest = 0;
}
    
// returns error code
int usage(int errcode);

// returns 0 on success
int split_path(char* dir, char* file, const char* path);

// returns 0 on success
int get_names(char* QParentClass, char* WidgetName, const char* filename);

// returns 0 on success
int run_uic(const char* uipath, const char* uiname, const char* widgetname);

// returns 0 on success
int run_moc(const char* uipath, const char* widgetname);

int main(int argc, char** argv)
{
  dbg();
  char uipath[256];
  char uiname[256];
  char WidgetName[256];
  char widgetname[256];
  char WIDGETNAME[256];
  char QParentClass[256];
  char outfile[256];
  FILE* fp;
  // usage error if no input param
  if(argc != 2)
    return usage(1);
  
  // usage ok if --help
  if(!strcmp(argv[1], "--help"))
    return usage(0);
  
  // split uipath, uiname
  if(split_path(uipath, uiname, argv[1]))
    return usage(1);
  
  // read WidgetName and QParentClass from ui file (else error)
  if(get_names(QParentClass, WidgetName, argv[1]))
  {
    usage(1);
    fprintf(stderr, 
            "Note: Can't find class and parent class defs in\n"
            "ui file: '%s'\n", argv[1]
           );
    return 1;
  }
  
  // generate widgetname and WIDGETNAME strings from WidgetName
  strupper(WIDGETNAME, WidgetName);
  strlower(widgetname, WidgetName);
   
  // write app-main.cpp, %s{widgetname}.cpp, and %s{widgetname}.h
  sprintf(outfile, "%s/app-main.cpp", uipath);
  fp = fopen(outfile, "w");
  if(!fp)
    error("Can't write %s", outfile);
  fprintf(fp, app_main_cpp, app_main_cpp_args);
  fclose(fp);
  
  sprintf(outfile, "%s/%s.cpp", uipath, widgetname);
  fp = fopen(outfile, "w");
  if(!fp)
    error("Can't write %s", outfile);
  fprintf(fp, widgetname_cpp, widgetname_cpp_args);
  fclose(fp);

  sprintf(outfile, "%s/%s.h", uipath, widgetname);
  fp = fopen(outfile, "w");
  if(!fp)
    error("Can't write %s", outfile);
  fprintf(fp, widgetname_h, widgetname_h_args);
  fclose(fp);
  
  if(run_uic(uipath, uiname, widgetname))
  {
    fprintf(stderr, "Error running uic on '%s'\n", uiname);
    return 1;
  }
  
  if(run_moc(uipath, widgetname))
  {
    fprintf(stderr, "Error running moc on '%s.h'\n", widgetname);
    return 1;
  }
  
  return 0;
}

// returns error code
int usage(int errcode)
{
#include "usagestr.dat"
  if(errcode)
    fprintf(stderr, "%s", usagestr);
  else
    fprintf(stdout, "%s", usagestr);
  return errcode;
}

// returns 0 on success
int split_path(char* dir, char* file, const char* path)
{
  const char* p = strchr(path, '/');
  char* dirptr = dir;
  if(p)
  {
    for(; path < p; path++)
      *dirptr++ = *path;
    *dirptr = 0;
    path++;
  }
  else
    strcpy(dirptr, ".");
  
  strcpy(file, path);
  return 0;
}

// returns 0 on success
int get_names(char* QParentClass, char* WidgetName, const char* filename)
{ 
  // it would be nice to be able to parse the xml file ourselves but 
  // we don't have a standard tool to do that yet, so we'll let QT do it
  // for us by capturing the uic output and filtering for one string 
  // that has both values we need, namely the 'setupUi()' function.
  
  // uic form1.ui | sed '/void setupUi/!d; s|void setupUi(||; s|)||; s|\*||' 
  char buf[4096];
  sprintf(buf, "uic '%s' | sed '/void setupUi/!d; s|void setupUi(||; s|)||; s|\\*||' 2>/dev/null", filename);
  FILE* fp = popen(buf, "r");
  if(!fp)
    return 1;
  *buf = 0;
  fgets(buf, 4096, fp);
  fclose(fp);
  
  ////////////////////
  char* ip = buf;
  char* op;
  
  // skip leading whitespaces if any
  while((*ip <= 32) && (*ip != 0))
    ip++;
  
  // get QParentClass while chars
  op = QParentClass;
  while(*ip > 32)
    *op++ = *ip++;
  *op = 0;
  
  // skip whitespaces
  while((*ip <= 32) && (*ip != 0))
    ip++;
  
  // and not end of input
  if(*ip == 0)
    return 1; // no widget name

  // get WidgetName while chars
  op = WidgetName;
  while(*ip > 32)
    *op++ = *ip++;
  *op = 0;
  
  // error if no WidgetName
  if(*WidgetName == 0)
    return 1;
  
  return 0;
}

// returns 0 on success
int run_uic(const char* uipath, const char* uiname, const char* widgetname)
{
  char cmd[256];
  sprintf(cmd, "uic %s/%s -o %s/ui_%s.h 2>/dev/null", uipath, uiname, uipath, widgetname);
  return system(cmd);
}

// returns 0 on success
int run_moc(const char* uipath, const char* widgetname)
{
  char cmd[256];
  sprintf(cmd, "moc %s/%s.h -o %s/moc_%s.cpp 2>/dev/null", uipath, widgetname, uipath, widgetname);
  return system(cmd);
}
