// qt4-ui.cpp
/*
  Outputs a ui: target for a makefile based on the probable 
  operations on files found that match criteria for 
    uic
    moc
    rcc
  that can be added to an mc2.def file to automatically build those files.
  
*/ 

#include <stdio.h>  // printf()
#include <string.h> // strncpy()
#include <stdlib.h> // getenv()

#include <LQ/slist.h>
#include <LQ/tmpstr.h>

void dbg(){}



int usage(int errcode);

void get_uiclist(char*** pfiles, char* path); // *.ui
void get_uicable(char*** puicables, char** files);
void get_moclist(char*** pfiles, const char* path); // *.h
void get_mocables(char*** pmocables, char** files);
void get_qrclist(char*** pfiles, char* path); // *.qrc
void get_qrcables(char*** pqrcables, char** files);

void print_ui_noop();
void print_ui_head();
void print_ui_foot(const char* path);


void print_uic(char** uicables, const char* path);
void print_moc(char** mocables, const char* path);
void print_qrc(char** qrcables, const char* path);


char HERE[256];

int main(int argc, char** argv)
{
  dbg();
  strncpy(HERE, getenv("PWD"), 256);
  if(argc != 2)
    return usage(1);
  if(!strcmp(argv[1], "--help"))
    return usage(0);
  
  char* path = argv[1];
      
  char** uicables = slist_new();
  char** mocables = slist_new();
  char** qrcables = slist_new();
  char** files = slist_new();
  
  get_uiclist(&files, path); // *.ui
  get_uicable(&uicables, files);
  int n_uic = slist_count(uicables);
  
  get_moclist(&files, path); // *.h
  get_mocables(&mocables, files);
  int n_moc = slist_count(mocables);
    
  get_qrclist(&files, path);  // *.qrc
  get_qrcables(&qrcables, files);
  int n_qrc = slist_count(qrcables);
      
  if((n_uic + n_moc + n_qrc) == 0)
  {
    print_ui_noop();
    return 0;
  }
  print_ui_head();
  if(n_uic)
    print_uic(uicables, path);
  if(n_moc)
    print_moc(mocables, path);
  if(n_qrc)
    print_qrc(qrcables, path);
  print_ui_foot(path);
  return 0;
}

// returns errcode
int usage(int errcode)
{
#include "usage_str.dat"

  if(errcode)
    fprintf(stderr, "%s", usage_str);
  else
    fprintf(stdout, "%s", usage_str);
  return errcode;
}

void print_ui_noop()
{
  printf("\nui: # noop\n\n");
}

void get_uiclist(char*** pfiles, char* path) // *.ui
{
  slist_clear(pfiles);
  char* cmd = tmpstr(256, "ls %s/*.ui 2>/dev/null", path);
  slist_pipeRead(pfiles, cmd);  
}

void get_uicable(char*** puicables, char** files)
{
  for(int i = 0; i < slist_count(files); i++)
  {
    const char* file = files[i];
    char* cmdstr = tmpstr(256, "uic %s >/dev/null 2>&1", file);
    if(!system(cmdstr))
    {
      slist_append(puicables, file);
      printf("# Found for uic: %s\n", file);
    }
    else
      printf("# Found for uic: %s but it appears to have problems\n", file);
    
  }
}

void get_moclist(char*** pfiles, const char* path) // *.h
{
  slist_clear(pfiles);
  char* cmd = tmpstr(256, "grep Q_OBJECT %s/*.h 2>/dev/null | cut -f1 -d:", path);
  slist_pipeRead(pfiles, cmd);  
}

void get_mocables(char*** pmocables, char** files)
{
  for(int i = 0; i < slist_count(files); i++)
  {
    const char* file = files[i];
    char* cmdstr = tmpstr(256, "moc %s >/dev/null 2>&1", file);
    if(!system(cmdstr))
    {
      printf("# Found for moc: %s\n", file);
      slist_append(pmocables, file);
    }
    else
      printf("# Found for moc: %s but it appears to have problems\n", file);
  }
}

void get_qrclist(char*** pfiles, char* path) // *.qrc
{
  slist_clear(pfiles);
  char* cmd = tmpstr(256, "ls %s/*.qrc 2>/dev/null", path);
  slist_pipeRead(pfiles, cmd);  
}

void get_qrcables(char*** pqrcables, char** files)
{
  for(int i = 0; i < slist_count(files); i++)
  {
    const char* file = files[i];

    char* cmdstr = tmpstr(256, "rcc %s >/dev/null 2>&1", file);
    if(!system(cmdstr))
    {
      slist_append(pqrcables, file);
      printf("# Found for qrc: %s\n", file);
    }
    else
      printf("# Found for qrc: %s but it appears to have problems\n", file);
  }
}


void print_ui_head()
{
  printf(
    "\n"
    "# These are the files that appear to be built by QT's tools.\n"
    "# Add include paths (-I <path> ) as needed.\n"
    "\n"
    "ui:\n"
  );

}

void print_ui_foot(const char* path)
{
  printf(
    "  mc2 -update # add new files to the makefile\n\n"    
    "clean-ui:\n"
    "  rm -f %s/ui_*.h %s/moc_*.cpp %s/qrc_*.cpp\n\n", path, path, path
  );
}

void print_uic(char** uicables, const char* path)
{
  for(int i = 0; i < slist_count(uicables); i++)
  {
    char* bname = tmpstr(256, uicables[i]);
    strrchr(bname, '.')[0] = 0;
    bname = strchr(bname, '/') + 1;
    printf("  uic %s/%s.ui -o %s/ui_%s.h\n", path, bname, path, bname);
  }
}

void print_moc(char** mocables, const char* path)
{
  for(int i = 0; i < slist_count(mocables); i++)
  {
    char* bname = tmpstr(256, mocables[i]);
    strrchr(bname, '.')[0] = 0;
    bname = strchr(bname, '/') + 1;
    printf("  moc %s/%s.h -o %s/moc_%s.cpp\n", path, bname, path, bname);
  }
}

void print_qrc(char** qrcables, const char* path)
{
  for(int i = 0; i < slist_count(qrcables); i++)
  {
    char* bname = tmpstr(256, qrcables[i]);
    strrchr(bname, '.')[0] = 0;
    bname = strchr(bname, '/') + 1;
    printf("  rcc %s/%s.qrc -o %s/qrc_%s.cpp # if compiled\n", path, bname, path, bname);
  }
}

