// mc2-writer.h

#ifndef mc2_writer_h
#define mc2_writer_h

#include <stdio.h> // FILE*

typedef struct
{
  const char* compiler_name;  /// typically gcc or g++
  const char* outname;        /// name of output binary
  const char* srcdir;         /// dir for sources
  const char* hdrdir;         /// dir for headers
  const char* objdir;         /// dir for temp obj files
  const char* bindir;         /// dir for output binary lib or executable
  char*** psrclist;           /// list of source files
  char*** phdrlist;           /// list of header files
  char*** pobjlist;           /// list of object files
  char*** psubdirs;           /// list of dirs for recursive make
}MC2_VARS;

/// writes the makefile with initialized vars
int write_makefile(MC2_VARS* o);

/// writes the mc2.def with env strings or defaults
int write_mc2_def(MC2_VARS* o);

/// creates missing dirs and loads file lists
void setup_main(MC2_VARS* o);

/// write the definition of PROJECT_FILES and MAKEFILES list definitions
void write_project_file_lists(FILE* fp, MC2_VARS* o);

/// write the target init-update: $(PROJECT_FILES)
void write_project_targets(FILE* fp, MC2_VARS* o);

/// write the rules per subdir for make, starting with all: tag
void write_project_rules(FILE* fp, MC2_VARS* o);

/// check if already defined in environment by user or makefile parser
const char *check_override(const char *varname, const char *default_val);

/// make corrections to certain switches in makefile that are easy to
/// mix up for beginners.
void check_end_switch(char *var, const char *endswitch1,
                      const char *endswitch2);

/// Writes a type NONE mc2.def for including non-mc2 makefiles to 
/// an mc2 PROJECT build.
void write_NONE_type();


#endif // mc2_writer_h

