// utils.h

#ifndef utils_h
#define utils_h

#include <malloc.h>		// mem
#include <string.h>		// str funcs

extern const char *appname;
extern const char *pwd;    // initial dir

// these return 0 on success else error

/// creates directories needed by the Makefile definitions, 
/// returns 0 on success.
int create_dirs(const char *srcdir, const char *objdir, const char *bindir);

/// creates list of source files, returns 0 on success.
int read_srclist(char ***psrclist, const char *srcdir);

/// creates list of header files, returns 0 on success.
int read_hdrlist(char ***phdrlist, const char *srcdir);

/// creates list of object files from source files, returns 0 on success.
int create_objlist(char ***pobjlist, char ***psrclist);

/// find mc2.def files in dirs one level down for recursive 'make', returns 0
/// on success
int create_subdirlist(char ***psubdirs);

/// prints message to stderr and aborts
void doerr(const char *msg);

/// true if mc2.def file is in current directory
bool have_mc2_def();

/// sets absolute path to the executable in buf, returns 0 on success
int setExecDir(char* buf, int buflen);

/// Prints character ch n times followed by a newline, to stdout or 
/// any other FILE*
void fprint_hline(FILE* fp, char ch, int n);

/// Takes a quoted comma separated list of buttons (minus spaces),
/// a message and possible extended xmessage commands.  Prints the
/// message in a centered window and returns the label of the button 
/// clicked as a recycling tmpstr() string that must be used before 
/// 32 new ones are created.
char* msgbox(const char* buttons, const char* msg, const char* ext);

extern const int mc2_dir_max; // = 256
extern char mc2_dir[256];    // execdirmax


// returns non-zero if targstr exists as a target in filename
int find_target(const char* targstr, const char* filename);

#endif // utils_h
