
#include <libgen.h>   // dirname()
#include <unistd.h>   // chdir(), get_current_dir_name()
#include <stdlib.h>   // setenv()
#include <string.h>   // strcmp()
#include <stdio.h>    // printf()
#include <sys/stat.h> // stat
#include <pwd.h>      // the passwd struct

#include <LQ/slist.h> 

void dbg(){}

char scratch[4096];

char* get_current_dir_name(); /// add this is you compile it as C

const char* getown(const char* filename); // # filename

int main(int argc, char** argv)
{
  dbg();
  // cd `dirname "$0"`  
  char* tmp = dirname(argv[0]);
  chdir(tmp);
  // cd ..
  chdir("..");
    
  // HERE=$PWD
  setenv("PWD", get_current_dir_name(), 1);
#define PWD getenv("PWD")
  setenv("HERE", PWD, 1);
#define HERE getenv("HERE")
  
  // if [ "$1" != "working" ];then ...
  if((argc < 2) || (strcmp(argv[1], "working") != 0)) {
    printf("\n"
        "  Only run this from the makefile as 'make <app>' where <app>\n"
        "  is the name of the function minus the 'mktools' prefix.\n"
        "\n"
        "  Press ENTER to abort now.\n"
        "\n"
        );
    // read key; exit
    getchar();  exit(0);
  } // fi
  
#define PREFIX getenv("PREFIX")
#define HOME getenv("HOME")
  
  // if [ "$PREFIX" == "$HOME" ]; then
  if(PREFIX && strcmp(PREFIX, HOME) == 0) {
  // exit 0; # not critical  
    exit(0); // not critical
  } // fi

  // # if running as root, assume all dirs and files belong to 
  // # root (UID=0) and issue warning if not

#define USER getenv("USER")
#define UID getuid()
  
  //  if [ "$UID" == "0" ]; then 
  //    username="root"
  //      else
  //      username="$USER"
  
  if(UID == 0) {
    setenv("username", "root", 1);
  }else{
    setenv("username", USER, 1);
  } // fi
  
#define username getenv("username")
  
  char** ilist = slist_new();
  
  // for i in $(cat files.dat dirs.dat); do
  slist_pipeRead(&ilist, "cat files.dat dirs.dat");
  for(int i = 0; i < slist_count(ilist); i++)
  {
    //  owner=`getown "TREE/$i"`
    sprintf(scratch, "TREE/%s", ilist[i]);
    const char* owner = getown(scratch);
    
    //  if [ "$owner" != "$username" ]; then
    if(strcmp(owner, username)) 
    {
    
      // echo " ...
      printf("\n"
        "Checking: %s\n"
        "File Owner: $owner\n"
        "Current User: $username\n"
        "\n"
        "WARNING:\n"
        "\n"
        "  Some files in the current TREE do not belong to '$username'\n"
        "\n"
        "  Recommend checking ownerships and changing them to '$username' \n"
        "  before installing or installing as '$username' as may be appropriate.\n"
        "\n"
        "  Press Ctrl-C to Abort or ENTER to continue.\n"
        "\n", ilist[i]
      );
      
      //  read key 
      //  exit
      getchar();
      exit(0);
    } // fi
  } //done
  
  return 0;
}


// returns a temp string, only valid until next call
const char* getown(const char* filename) // # filename
{
  //  local filename=$1
  //      echo $(stat $filename) | cut -f2 -d')' | sed 's|.* ||'
  
  static char* s; // the return string
  int err = 0;
  struct stat st;
  err = stat(filename, &st);
  if(!err)
  {
    struct passwd* pw_ent = getpwuid (st.st_uid);
    if(pw_ent->pw_name)
      strcpy(scratch, pw_ent->pw_name);
    else 
      err = 1;
  }
  if(err)
    strcpy(scratch, "UNKNOWN");
  
  if(s) free(s);
  s = strdup(scratch);
  return s;
}

